#  Copyright (c) 2010 Vitja Makarov
#
#  Author: Vitja Makarov <vitja.makarov@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os, sys

import apt
import apt_pkg

from uirk.project import *
from uirk.pkglist import PackageList
from uirk import utils
from utils import cached_property
from uirk.task import ExecTask
from uirk.i18n import gettext as _

def dependencies(cache, pkg, deps, key="Depends"):
    """See: python-apt/doc/examples/all_deps.py"""
    candver = cache._depcache.GetCandidateVer(pkg._pkg)
    if candver is None:
        return deps
    dependslist = candver.DependsList
    deps[pkg.name] = apt.Package(cache, pkg._pkg)
    if key in dependslist:
        for depVerList in dependslist[key]:
            for dep in depVerList:
                if dep.TargetPkg.Name in cache:
                    if pkg.name != dep.TargetPkg.Name and \
                        not deps.has_key(dep.TargetPkg.Name):
                        deps[dep.TargetPkg.Name] = apt.Package(cache, dep.TargetPkg)
                        dependencies(
                            cache, cache[dep.TargetPkg.Name], deps, key)
    return deps

def dependencies_for(cache, pkgs, deps=None, keys=("Depends", "PreDepends")):
    """Calculate dependencies for list of packages"""
    if deps is None:
        deps = {}
    for pkg in pkgs:
        for key in keys:
            dependencies(cache, pkg, deps, key)
    return deps

class PoolDeps(object):
    """Pool dependencies"""
    def __init__(self, prj):
        self.prj = prj
        self.deps = {}

    @cached_property
    def md5_cache(self):
        path = os.path.join(self.prj.pool_cache_dir, '.md5cache')
        from md5cache import MD5Cache
        return MD5Cache(path)

    def update_md5_cache(self):
        def isnt_deb(p):
            return not p.endswith(".deb")
        self.md5_cache.md5sum(self.prj.pool_cache_dir, exclude=isnt_deb)
        if self.md5_cache.dirty:
            self.md5_cache.write()

    def copy_cached(self):
        """Copy (hard-link) cached packages from target fs into local cache"""
        archives = os.path.join(self.prj.fsdir, 'var/cache/apt/archives')
        for fname in os.listdir(archives):
            path = os.path.join(archives, fname)
            if not os.path.isfile(path) or not fname.endswith('.deb'):
                continue
            pool_path = os.path.join(self.prj.pool_cache_dir, fname)
            if os.path.exists(pool_path):
                if utils.same_inode(path, pool_path):
                    continue
            os.link(path, pool_path)

    def add_pkgnames(self, lst):
        """Add package names into dependency list"""
        keys=["Depends", "PreDepends"]
        if self.prj.pool_include_recommends:
            keys.append("Recommends")
        if self.prj.pool_include_suggests:
            keys.append("Suggests")
        pkgs = set()
        for name in lst:
            if self.prj.apt_cache.has_key(name):
                pkg = [self.prj.apt_cache[name]]
            else:
                pkg = self.prj.apt_cache.getProvidingPackages(name)
            pkgs.update(pkg)
        dependencies_for(self.prj.apt_cache, pkgs, self.deps, keys=keys)

    def __iter__(self):
        return self.deps.itervalues()

class PoolFetcher(object):
    """Fetches pool entries into cache"""
    def __init__(self, pool):
        self.pool = pool

    def run(self, progress=None):
        acq = apt_pkg.Acquire(progress)
        items = []
        self.pool.copy_cached()
        self.pool.update_md5_cache()
        for pkg in self.pool:
            if pkg.isInstalled and not self.pool.prj.pool_include_installed:
                continue
            candidate = pkg.candidate
            base = os.path.basename(candidate._records.filename)
            if self.pool.md5_cache.has_key(base):
                if candidate.md5 == self.pool.md5_cache[base].md5:
                    continue
            destfile = os.path.join(self.pool.prj.pool_cache_dir, base)
            f = apt_pkg.AcquireFile(acq, candidate.uri, candidate._records.md5_hash,
                                              candidate.size, base, destfile=destfile)
            items.append(f)
        acq.run()

class DebDoesNotExists(Exception):
    pass


class AptFtpArchive(ExecTask):
    def __init__(self, prj, components):
        from tempfile import NamedTemporaryFile

        self.conf = NamedTemporaryFile()

        self.conf.write('APT::FTPArchive::Release::Origin "Ubuntu";\n'
                    'APT::FTPArchive::Release::Label "Ubuntu";\n'
                    'APT::FTPArchive::Release::Suite "%(codename)s";\n'
                    'APT::FTPArchive::Release::Version "%(version)s";\n'
                    'APT::FTPArchive::Release::Codename "%(codename)s";\n'
                    'APT::FTPArchive::Release::Components "%(components)s";\n'
                    'APT::FTPArchive::Release::Architectures "%(arch)s";\n'
                    'APT::FTPArchive::Release::Description "%(description)s";\n'
                    % dict(codename=prj.distrib_codename,
                            version=prj.distrib_release,
                            components=' '.join(components),
                            arch=prj.arch,
                            description=prj.distrib_description))
        self.conf.flush()

        args = [os.path.join(utils.SCRIPTS_DIR, 'mkpool.sh'),
                        prj.diskdir, self.conf.name, prj.arch, prj.distrib_codename,
                        os.path.join(prj.pool_cache_dir, 'ftparchive.db')]
        args.extend(components)
        super(AptFtpArchive, self).__init__(args, message=_("Creating ftp archive"))

class PoolTreeGenerator(object):
    def __init__(self, pool):
        self.prj = pool.prj
        self.pool = pool

    def run(self, silent_fail=True, progress=None):
        pooled = set()
        pooldir = os.path.join(self.prj.diskdir, 'pool')
        components = set()
        for pkg in self.pool:
            if pkg.isInstalled and not self.pool.prj.pool_include_installed:
                continue
            candidate = pkg.candidate
            ## XXX: should guess it from package
            component = pkg.section.split('/')
            if len(component) != 2:
                component = 'main'
            else:
                component = component[0]
            components.add(component)
            if pkg.sourcePackageName.startswith('lib'):
                short = pkg.sourcePackageName[:4]
            else:
                short = pkg.sourcePackageName[:1]
            filename = os.path.basename(candidate.filename)
            path = os.path.join(self.prj.diskdir, 'pool/%s/%s/%s' % (component, short, pkg.sourcePackageName))
            dst_path = os.path.join(path, filename)
            utils.makedirs_safe(path)

            src_path = os.path.join(self.pool.prj.pool_cache_dir,  filename)

            if not os.path.exists(src_path):
                if silent_fail:
                    print 'Warning! file %s does not exists' % filename
                    continue
                raise DebDoesNotExists, "Deb file does not exists %s" % filename

            pooled.add(dst_path)
            if os.path.exists(dst_path):
                if utils.same_inode(src_path, dst_path):
                    continue
                os.unlink(dst_path)
            print '%s -> %s' % (src_path, dst_path)
            os.link(src_path, dst_path)

        # cleanup
        for root,dirs,files in os.walk(pooldir):
            items = len(dirs) + len(files)
            for fname in files:
                full_path = os.path.join(root, fname)
                if full_path in pooled:
                    continue
                os.unlink(full_path)
                items -= 1
            if items == 0:
                os.rmdir(root)

        AptFtpArchive(self.prj, components).run(progress=progress)

        if self.prj.gpg_key:
            r = os.system('gpg --yes -u %(key)s --output %(fname)s.gpg -ba %(fname)s' % dict(key=self.prj.gpg_key,
                        fname=os.path.join(self.prj.diskdir, 'dists', self.prj.distrib_codename, 'Release')))
            if r != 0:
                raise utils.OperationError, _("GPG exited with %d code") % r
        else:
            print 'Warning! No GPG key specified can not sign debs'
