#  Copyright (c) 2010 Vitja Makarov
#
#  Author: Vitja Makarov <vitja.makarov@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os, sys

from uirk.progress import base
from uirk import utils
from uirk.i18n import gettext as _

from apt.progress.text import TextProgress

class BaseTextProgress(object):
    def __init__(self, *args, **kwargs):
        self._progressbar_width = kwargs.pop('progressbar_width', 40)
        self._debug_log = []
        self._cntr = 0
        self.sudo_cmd = utils.SUDO
        super(BaseTextProgress, self).__init__(*args, **kwargs)

    def counter(self):
        self._cntr = (self._cntr + 1)& 3
        return '|/-\\'[self._cntr]

    def progress_bar(self, percents):
        a = min(int(percents * self._progressbar_width / 100), self._progressbar_width)
        b = max(0, self._progressbar_width - a)
        if b > 0:
            c = self.counter()
            b -= 1
        else:
            c = ''
        return '[' + '#' * a + c + ' ' * b + ']'

    def handle_status_line(self, line):
        self.log_verbose(line)
        super(BaseTextProgress, self).handle_status_line(line)

    def complete(self):
        if not self.verbose:
            self.update_percents(100)
            sys.stdout.write('\n')

    def error(self, exit_code):
        sys.stdout.write('\n')
        for i in self._debug_log:
            sys.stdout.write('...' + i)
        if os.WIFEXITED(exit_code):
            sys.stdout.write(_('Program exited with code %d\n') % os.WEXITSTATUS(exit_code))
        elif os.WIFSIGNALED(exit_code):
            sys.stdout.write(_('Program was killed by signal %d\n') % (exit_code & 0xff))
        else:
            sys.stdout.write(_('Program terminated with status code %d\n') % exit_code)

    def log_verbose(self, data):
        if data.endswith('\n'):
            self._debug_log.append(data)
        if self.verbose:
            sys.stdout.write(data)
            sys.stdout.flush()

class TextBuildISOProgress(BaseTextProgress, base.BaseBuildISOProgress):
    def __init__(self, *args, **kwargs):
        super(TextBuildISOProgress, self).__init__(*args, **kwargs)

    def update_percents(self, percents):
        if self.verbose:
            return
        sys.stdout.write(_('\rBuilding ISO filesystem: %s % 6.2lf%%') % (self.progress_bar(percents), percents))
        sys.stdout.flush()

class TextSquashfsProgress(BaseTextProgress, base.BaseSquashfsProgress):
    def __init__(self, *args, **kwargs):
        super(TextSquashfsProgress, self).__init__(*args, **kwargs)
        self.started = False

    def handle_timeout(self):
        if self.verbose:
            return

    def update_percents(self, percents):
        if self.verbose:
            return
        if percents > 0:
            self.started = True
        if self._unsquashfs:
            sys.stdout.write(_('\rExtracting squashfs: %s % 6.2lf%%') % (self.progress_bar(percents), percents))
        else:
            sys.stdout.write(_('\rCreating squashfs: %s % 6.2lf%%') % (self.progress_bar(percents), percents))
        sys.stdout.flush()

class TextExtractISOProgress(BaseTextProgress, base.BaseExtractISOProgress):
    def __init__(self, *args, **kwargs):
        super(TextExtractISOProgress, self).__init__(*args, **kwargs)
        self.started = False

    def handle_timeout(self):
        if self.verbose:
            return
        if self.started:
            sys.stdout.write(_('\rExtracting ISO image: %s') % self.counter())
            sys.stdout.flush()

    def handle_status_line(self, data):
        line = data.rstrip()
        if line == "STARTING":
            self.started = True
        else:
            self.log_verbose(data)
        if self.started:
            self.handle_timeout()

    def complete(self):
        sys.stdout.write(_('\rExtracting ISO image: DONE\n'))

class TextInitrdUpdateProgress(BaseTextProgress, base.BaseInitrdUpdateProgress):
    def handle_timeout(self):
        if self.verbose:
            return
        sys.stdout.write(_('\rUpdating initrd image: %s') % self.counter())
        sys.stdout.flush()

    def handle_status_line(self, data):
        self.log_verbose(data)
        self.handle_timeout()

    def complete(self):
        sys.stdout.write(_('\rUpdating initrd image: DONE\n'))

class TextExecTaskProgress(BaseTextProgress, base.BaseExecTaskProgress):
    def handle_timeout(self):
        if self.verbose:
            return
        sys.stdout.write('\r%s: %s' % (self.message, self.counter()))
        sys.stdout.flush()

    pulse = handle_timeout

    def handle_status_line(self, data):
        self.log_verbose(data)
#        self.handle_timeout()

    def complete(self):
        if not self.verbose:
            sys.stdout.write('\r%s: %s\n' % (self.message, _('DONE')))
