#  Copyright (c) 2010 Vitja Makarov
#
#  Author: Vitja Makarov <vitja.makarov@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__all__ = ["KNOWN_ARCHES", "INITRD_TYPES", "DiskInfo", "RemasterProject", "DiskInfoError", "RemasterProjectError"]
import os.path
import ConfigParser

import apt

from uirk import diskdefines
from uirk.utils import cached_property
from uirk import utils
from uirk.md5cache import MD5Cache
from uirk.simplecfg import SimpleConfigParser
from uirk.i18n import gettext as _

KNOWN_ARCHES = ('i386', 'amd64')
INITRD_EXTS = {
    'gzip': 'gz',
    'lzma': 'lz',
}
INITRD_TYPES = INITRD_EXTS.keys()

class Error(Exception):
    pass

class DiskInfoError(Exception):
    """Error fetching disk info"""

class DiskInfo(object):
    """Fetch generic info about disk"""

    def __init__(self, diskdir):
        self.squashfs_path = os.path.join(diskdir, 'casper/filesystem.squashfs')
        self.diskdefines_path = os.path.join(diskdir, 'README.diskdefines')

        if not os.path.isfile(self.squashfs_path):
            raise DiskInfoError, _("Can not find squashfs file on disk")
        if not os.path.isfile(self.diskdefines_path):
            raise DiskInfoError, _("Can not find README.diskdefines file on disk")

        defines = diskdefines.read(self.diskdefines_path)

        if not defines.has_key('ARCH'):
            raise DiskInfoError, _('README.diskdefines does not define architecture')
        self.arch = defines['ARCH']

        if not defines.has_key('DISKNAME'):
            self.diskname = 'unknown'
        else:
            diskname = defines['DISKNAME'].rsplit(' - ', 1)
            self.diskname = ''.join(diskname[:1])

        for (name, ext)  in INITRD_EXTS.items():
            self.initrd_path = os.path.join(diskdir, 'casper/initrd.%s' % ext)
            if os.path.exists(self.initrd_path):
                self.initrd_type = name
                break
        else:
            raise DiskInfoError, _("Can not find initrd image")

        self.isolinuxbin_path = os.path.join(diskdir, 'isolinux/isolinux.bin')
        self.isolinuxboot_path = os.path.join(diskdir, 'isolinux/boot.cat')

class RemasterProjectError(Exception):
    pass

class RemasterProject(object):
    def __init__(self, prjdir, diskinfo=None):
        self.prjdir = os.path.abspath(prjdir)
        self.fsdir = os.path.join(self.prjdir, "fs")
        self.diskdir = os.path.join(self.prjdir, "disk")
        self.poolcachedir = os.path.join(self.prjdir, ".pool-cache")
        self.config_fname = os.path.join(self.prjdir, "uirk.settings")
        self.config = ConfigParser.SafeConfigParser()

        self.label = 'Put your disk label here'
        self.arch = 'unknown'
        self.initrd_type = 'gzip'

        self.pool_include_installed = False
        self.pool_include_recommends = False
        self.pool_include_suggests = False

        self.locales = []

        self.gpg_key = None

        if diskinfo:
            self.arch = diskinfo.arch
            self.label = diskinfo.diskname
            self.initrd_type = diskinfo.initrd_type

        if os.path.exists(self.config_fname):
            self._load_config()
    def __repr__(self):
        return u'<RemasterProject: label="%s" arch="%s">' % (self.label, self.arch)

    def _load_config(self):
        fp = open(self.config_fname)
        self.config.readfp(fp)
        fp.close()

        if not self.config.has_section('diskinfo'):
            raise RemasterProjectError, \
                _("Can not find 'diskinfo' section in config")
        try:
            self.arch = self.config.get('diskinfo', 'arch')
        except ConfigParser.NoOptionError:
            raise RemasterProjectError, _("No arch defined in config")
        if self.arch not in KNOWN_ARCHES:
            raise RemasterProjectError, _("Unknown architecture %s") % self.arch

        try:
            self.initrd_type = self.config.get('diskinfo', 'initrd_type')
        except ConfigParser.NoOptionError:
            raise RemasterProjectError, _("No initrd type defined")

        try:
            self.label = self.config.get('diskinfo', 'label')
        except ConfigParser.NoOptionError:
            pass

        if self.config.has_section('pool'):
            try:
                self.pool_include_installed = self.config.getboolean('pool', 'include_installed')
            except (ConfigParser.NoOptionError, ValueError):
                pass
            try:
                self.pool_include_suggests = self.config.getboolean('pool', 'include_suggests')
            except (ConfigParser.NoOptionError, ValueError):
                pass
            try:
                self.pool_include_recommends = self.config.getboolean('pool', 'include_recommends')
            except (ConfigParser.NoOptionError, ValueError):
                pass

        if self.config.has_section('locale'):
            try:
                locales = self.config.get('locale', 'locales')
            except ConfigParser.NoOptionError:
                locales = ""
            self.locales = locales.split()

        if self.config.has_section('private'):
            try:
                self.gpg_key = self.config.get('private', 'gpg_key')
            except ConfigParser.NoOptionError:
                pass

    def write(self):
        """Write project configuration"""
        if not self.config.has_section('diskinfo'):
            self.config.add_section('diskinfo')
        self.config.set('diskinfo', 'label', self.label)
        self.config.set('diskinfo', 'arch', self.arch)
        self.config.set('diskinfo', 'initrd_type', self.initrd_type)
        if not self.config.has_section('pool'):
            self.config.add_section('pool')
        self.config.set('pool', 'include_installed', str(self.pool_include_installed))
        self.config.set('pool', 'include_recommends', str(self.pool_include_recommends))
        self.config.set('pool', 'include_suggests', str(self.pool_include_suggests))

        if not self.config.has_section('private'):
            self.config.add_section('private')
        if self.gpg_key:
            self.config.set('private', 'gpg_key', self.gpg_key)
        if not self.config.has_section('locale'):
            self.config.add_section('locale')
        self.config.set('locale', 'locales', ' '.join(self.locales))

        fp = open(self.config_fname, 'w')
        self.config.write(fp)
        fp.close()

    @cached_property
    def tmp_dir(self):
        """Directory where prepared linux kernel and initrd are templorary stored"""
        path = os.path.join(self.prjdir, '.tmp')
        utils.mkdir_safe(path)
        return path

    @cached_property
    def pool_cache_dir(self):
        """Directory where all the pool debs are stored"""
        path = os.path.join(self.prjdir, '.pool')
        utils.mkdir_safe(path)
        return path

    @cached_property
    def casper_dir(self):
        return os.path.join(self.diskdir, 'casper')

    @cached_property
    def vmlinuz_path(self):
        return os.path.join(self.diskdir, 'casper/vmlinuz')

    @cached_property
    def initrd_path(self):
        return os.path.join(self.diskdir, 'casper/initrd.%s' % \
                        INITRD_EXTS[self.initrd_type])

    @cached_property
    def squashfs_path(self):
        return os.path.join(self.diskdir, 'casper/filesystem.squashfs')

    @cached_property
    def casper_uuid_path(self):
        return os.path.join(self.diskdir, '.disk/casper-uuid-generic')

    @cached_property
    def fstimestamp_path(self):
        return os.path.join(self.prjdir, '.fs-timestamp')


    def update_fs_timestamp(self):
        """Timestamp file should be touched after each filesystem access"""
        utils.touch(self.fstimestamp_path)

    @property
    def fs_dirty(self):
        """Is filesystem dirty"""
        return utils.fileup(self.fstimestamp_path, self.squashfs_path)

    @cached_property
    def md5cache_path(self):
        return os.path.join(self.prjdir, '.md5cache')

    @cached_property
    def md5cache(self):
        return MD5Cache(self.md5cache_path)

#    def is_iso_dirty(self, fname):
#        return utils.fileup(fname, self.md5cache_path)

    @cached_property
    def apt_cache(self):
        import apt_pkg
        apt_pkg.init()
        apt_pkg.Config['APT::Architecture'] = self.arch
        return apt.Cache(rootdir=self.fsdir)

    @cached_property
    def lsb_realease_info(self):
        p = SimpleConfigParser(defaults=dict(DISTRIB_ID='Ubuntu',
            DISTRIB_RELEASE="unknown",
            DISTRIB_CODENAME="unknown",
            DISTRIB_DESCRIPTION="unknown"))
        p.read(os.path.join(self.fsdir, 'etc/lsb-release'))
        return p

    @cached_property
    def distrib_codename(self):
        return self.lsb_realease_info['DISTRIB_CODENAME']
    @cached_property
    def distrib_release(self):
        return self.lsb_realease_info['DISTRIB_RELEASE']
    @cached_property
    def distrib_description(self):
        return self.lsb_realease_info['DISTRIB_DESCRIPTION']
    @cached_property
    def distrib_id(self):
        return self.lsb_realease_info['DISTRIB_ID']

if __name__ == "__main__":
    prj = RemasterProject('.')
    print prj
    prj.write()
